@@ -31,6 +31,7 @@ int main(int argc, char *argv[])
 	struct rtnl_link *link;
 	struct nl_cache *link_cache;
 	struct nl_sock *sk;
+	struct rtnl_link *ltap;
 	int err;
 
 	sk = nl_socket_alloc();
@@ -52,7 +53,7 @@ int main(int argc, char *argv[])
 	nl_cache_refill(sk, link_cache);
 
 	link = rtnl_link_get_by_name(link_cache, TEST_BRIDGE_NAME);
-	struct rtnl_link *ltap = rtnl_link_get_by_name(link_cache, TEST_INTERFACE_NAME);
+	ltap = rtnl_link_get_by_name(link_cache, TEST_INTERFACE_NAME);
 	if (!ltap) {
 		fprintf(stderr, "You should create a tap interface before lunch this test (# tunctl -t %s)\n", TEST_INTERFACE_NAME);
 		return -1;
@@ -67,6 +68,11 @@ int main(int argc, char *argv[])
 		fprintf(stderr, "Link is not a bridge\n");
 		return -2;
 	}
+
+	rtnl_link_put(ltap);
+	nl_cache_refill(sk, link_cache);
+	ltap = rtnl_link_get_by_name(link_cache, TEST_INTERFACE_NAME);
+
 	if(rtnl_link_get_master(ltap) <= 0) {
 		fprintf(stderr, "Interface is not attached to a bridge\n");
 		return -3;
